﻿using GameUtils.Graphics.Data;
using SharpDX;
using SharpDX.Direct2D1;

namespace GameUtils.Renderers.Direct2D1_1
{
    internal class Direct2DTexture : TextureData
    {
        readonly Bitmap1 bmp;

        public Bitmap1 GetBitmap()
        {
            return bmp;
        }

        protected override int Width
        {
            get { return bmp.PixelSize.Width; }
        }

        protected override int Height
        {
            get { return bmp.PixelSize.Height; }
        }

        unsafe private static Bitmap1 ConvertBitmap(DeviceContext deviceContext, System.Drawing.Bitmap bitmap)
        {
            int stride = bitmap.Width * sizeof(int);
            using (var tempStream = new DataStream(bitmap.Height * stride, true, true))
            {
                var sourceArea = new System.Drawing.Rectangle(0, 0, bitmap.Width, bitmap.Height);
                var bitmapData = bitmap.LockBits(sourceArea, System.Drawing.Imaging.ImageLockMode.ReadOnly, System.Drawing.Imaging.PixelFormat.Format32bppPArgb);

                uint* p = (uint*)bitmapData.Scan0;
                for (int i = 0; i < bitmap.Width * bitmap.Height; i++)
                {
                    byte* channels = (byte*)p;
                    tempStream.Write(channels[2]);
                    tempStream.Write(channels[1]);
                    tempStream.Write(channels[0]);
                    tempStream.Write(channels[3]);
                    p++;
                }

                bitmap.UnlockBits(bitmapData);
                tempStream.Position = 0;

                var bitmapProperties = new BitmapProperties1(new PixelFormat(SharpDX.DXGI.Format.R8G8B8A8_UNorm, AlphaMode.Premultiplied));
                return new Bitmap1(deviceContext, new Size2(bitmap.Width, bitmap.Height), tempStream, stride, bitmapProperties);
            }
        }

        internal Direct2DTexture(DeviceContext deviceContext, string path)
        {
            using (var bitmap = new System.Drawing.Bitmap(path))
            {
                bmp = ConvertBitmap(deviceContext, bitmap);
            }            
        }

        internal Direct2DTexture(DeviceContext deviceContext, System.IO.Stream stream)
        {
            using (var bitmap = new System.Drawing.Bitmap(stream))
            {
                bmp = ConvertBitmap(deviceContext, bitmap);
            }            
        }

        protected override void Dispose(bool disposing)
        {
            bmp.Dispose();
        }
    }
}
