﻿using GameUtils.Graphics;
using GameUtils.Graphics.Data;
using GameUtils.Math;

namespace GameUtils.Renderers.Gdi
{
    internal class GdiFactory : Factory
    {
        protected override SolidColorBrushData CreateSolidColorBrush(Color4 color)
        {
            return new GdiSolidColorBrush(color);
        }

        protected override LinearGradientBrushData CreateLinearGradientBrush(GradientStop[] stops, Vector2 start, Vector2 end)
        {
            return new GdiLinearGradientBrush(stops, start, end);
        }

        protected override RadialGradientBrushData CreateRadialGradientBrush(GradientStop[] stops, Vector2 center, float radiusX, float radiusY)
        {
            return new GdiRadialGradientBrush(stops, center, radiusX, radiusY);
        }

        protected override TextureBrushData CreateTextureBrush(Texture texture)
        {
            return new GdiTextureBrush(texture);
        }

        protected override GeometryData CreateGeometry()
        {
            return new GdiGeometry();
        }

        protected override FontData CreateFont(string familyName, float size, FontStyle style, FontWeight weight)
        {
            return new GdiFont(familyName, size, style, weight);
        }

        protected override TextureData CreateTexture(string path)
        {
            return new GdiTexture(path);
        }

        protected override TextureData CreateTexture(System.IO.Stream stream)
        {
            return new GdiTexture(stream);
        }

        protected override TextFormatData CreateTextFormat()
        {
            return new GdiTextFormat();
        }
    }
}
