﻿using System.Drawing;
using System.Drawing.Drawing2D;
using GameUtils.Graphics.Data;
using GameUtils.Math;
using Drawing = System.Drawing.Drawing2D;
using GameUtils.Graphics;

namespace GameUtils.Renderers.Gdi
{
    internal class GdiLinearGradientBrush : LinearGradientBrushData, IGdiBrush
	{
		Drawing.LinearGradientBrush brush;
		GradientStop[] stops;
		float opacity;
		Vector2 start;
		Vector2 end;
        Matrix matrix;

        public Brush Brush
        {
            get { return brush; }
        }

        protected override GradientStop[] GradientStops
		{
			get { return stops; }
			set
			{
				stops = value;
				brush.Dispose();
				brush = CreateBrush();
			}
		}

        protected override float Opacity
		{
			get { return opacity; }
			set
			{
				if (value > 1)
					opacity = 1;
				else if (value < 0)
					opacity = 0;
				else
					opacity = value;
				brush.Dispose();
				brush = CreateBrush();
			}
		}

        protected override Vector2 StartPoint
		{
			get { return start; }
			set
			{
				start = value;
				brush.Dispose();
				brush = CreateBrush();
			}
		}

        protected override Vector2 EndPoint
		{
			get { return end; }
			set
			{
				end = value;
				brush.Dispose();
				brush = CreateBrush();
			}
		}

		private Drawing.LinearGradientBrush CreateBrush()
		{
			var b = new Drawing.LinearGradientBrush(GdiHelper.Vector2ToPointF(start), GdiHelper.Vector2ToPointF(end), Color.Transparent, Color.Transparent);
			var colors = new Color[stops.Length];
			var positions = new float[stops.Length];
			for (int i = 0; i < stops.Length; i++)
			{
				var color = stops[i].Color;
				color.A *= opacity;
				colors[i] = GdiHelper.ConvertColor(color);
				positions[i] = stops[i].Position;
			}
			var blend = new ColorBlend();
			blend.Colors = colors;
			blend.Positions = positions;
			b.InterpolationColors = blend;
            //b.WrapMode = Drawing.WrapMode.Clamp;
		    if (matrix != null) b.Transform = matrix;
			return b;
		}

		internal GdiLinearGradientBrush(GradientStop[] stops, Vector2 start, Vector2 end)
		{
			this.stops = stops;
			opacity = 1;
			this.start = start;
			this.end = end;
			brush = CreateBrush();
		}

        protected override void SetTransform(Matrix2x3 matrix)
        {
            var gdiMatrix = new Matrix(matrix[0, 0], matrix[1, 0], matrix[0, 1], matrix[1, 1], matrix.OffsetX, matrix.OffsetY);
            brush.Transform = gdiMatrix;
            if (this.matrix != null)
                this.matrix.Dispose();
            this.matrix = gdiMatrix;
        }

        protected override void ResetTransform()
        {
            brush.ResetTransform();
            if (matrix != null)
                matrix.Dispose();
            matrix = null;
        }

		protected override void Dispose(bool disposing)
		{
			brush.Dispose();
            if (matrix != null) matrix.Dispose();
		}
    }
}
