﻿using System.Drawing;
using GameUtils.Graphics;
using GameUtils.Graphics.Data;

namespace GameUtils.Renderers.Gdi
{
    internal class GdiTextFormat : TextFormatData
    {
        readonly StringFormat format;
        WordWrapping wordWrapping;

        public StringFormat GetStringFormat()
        {
            return format;
        }

        protected override HorizontalAlignment HorizontalAlignment
        {
            get { return (HorizontalAlignment)format.Alignment; }
            set { format.Alignment = (StringAlignment)value; }
        }

        protected override VerticalAlignment VerticalAlignment
        {
            get { return (VerticalAlignment)format.LineAlignment; }
            set { format.LineAlignment = (StringAlignment)value; }
        }

        protected override WordWrapping WordWrapping
        {
            get { return wordWrapping; }
            set
            {
                wordWrapping = value;
                switch (wordWrapping)
                {
                    case WordWrapping.NoWrap:
                        format.FormatFlags |= StringFormatFlags.NoClip;
                        format.FormatFlags |= StringFormatFlags.NoWrap;
                        break;
                    case WordWrapping.Wrap:
                        format.FormatFlags &= ~StringFormatFlags.NoClip;
                        format.FormatFlags &= ~StringFormatFlags.NoWrap;
                        break;
                    case WordWrapping.Clip:
                        format.FormatFlags &= ~StringFormatFlags.NoClip;
                        format.FormatFlags |= StringFormatFlags.NoWrap;
                        break;
                }
            }
        }

        internal GdiTextFormat()
        {
            format = new StringFormat(StringFormat.GenericDefault);
        }

        protected override void Dispose(bool disposing)
        {
            format.Dispose();
        }
    }
}
